#!/bin/bash -

# Purpose: SPKAC (Signed Public Key and Challenge) certificate request
# 9 Arguments: 1st: name of the file without extension, 2nd: Challenge and the certificate attributes (C, ST, L, O, OU, CN and Email)
RequestedArgs=9

#undefine alias
\unalias -a

#disable history
hash -r

# Setting environment
BASEDIR=$(dirname $0)
if [ -f $BASEDIR/pki.conf ]; then
	. $BASEDIR/pki.conf

	if [ -z "$APPBASE" ] || [ -z "$PKIDIR" ] || [ -z "$CONFDIR" ] || [ -z "$LOGDIR" ] || [ -z "$BINDIR" ]; then
		printf "%b" "APPBASE, PKIDIR, LOGDIR, BINDIR and CONFDIR variables must be defined in \"$BASEDIR/pki.conf\"\n" >&2
		printf "%b" "PATH $PATH\n" >&2
		exit 1
	fi

else	
	printf "%b" "Error. File \"$BASEDIR/pki.conf\" not found.\n" >&2
        exit 2
fi


# Going to the PKI directories
cd $PKIDIR

# Analysing the arguments
if (( $# < $RequestedArgs ))
then
        printf "%b" "Error. Not enough arguments.\n" >&2
        printf "%b" "Usage : $0 filename challenge C ST L O OU CN Email\n" >&2
        exit 1
elif (( $# > $RequestedArgs ))
then
        printf "%b" "Error. Too many arguments.\n" >&2
        printf "%b" "usage : $0 filename challenge C ST L O OU CN Email\n" >&2
        exit 2
else
        printf "%b" "In progress ..."
fi


# Generating the SPKAC certificate request

# Generating the key pair
echo -e "\n\nGenerating the RSA private and public key ..."
openssl genrsa -aes256 -out $PKIDIR/private/${1}.key -rand $PKIDIR/private/.rand 2048
 
# Protecting the private Key
echo -e "\n\nSetting ACL for the private Key ..."
chgrp wheel $PKIDIR/private/${1}.key
chmod 440 $PKIDIR/private/${1}.key
 
# Generating SPKAC file
echo -e "\n\nGenerating the SPKAC File ..."
openssl spkac -key $PKIDIR/private/${1}.key -challenge ${2} -out $PKIDIR/csr/${1}.spkac

# Adding LDAP data for the Distinguished Name (DN)
echo -e "\n\nAdding LDAP Attributes to the SPKAC File for the Distinguished Name (DN)..."
echo C=${3} >> $PKIDIR/csr/${1}.spkac
echo ST=${4} >> $PKIDIR/csr/${1}.spkac
echo L=${5} >> $PKIDIR/csr/${1}.spkac
echo O=${6} >> $PKIDIR/csr/${1}.spkac
echo OU=${7} >> $PKIDIR/csr/${1}.spkac
echo CN=${8} >> $PKIDIR/csr/${1}.spkac
echo emailAddress=${9} >> $PKIDIR/csr/${1}.spkac

# Checking the signature
echo -e "\n\nChecking the signature ..."
openssl spkac -in $PKIDIR/csr/${1}.spkac -noout -verify

# Extracting the public key
echo -e "\n\nExtracting the Public Key from SPKAC ..."
openssl spkac -in $PKIDIR/csr/${1}.spkac -noout -pubkey -out $PKIDIR/private/${1}.pub.key

# Printing SPKAC content
echo -e "\n\nPrinting SPKAC content ..."
openssl spkac -in $PKIDIR/csr/${1}.spkac

# Signing the certificate
echo -e "\n\nSigning the certificate from SPKAC ..."
openssl ca -config $CONFDIR/ca-user-cert.cnf -days 365 -spkac $PKIDIR/csr/${1}.spkac -notext > $PKIDIR/certs/${1}.crt

echo -e "\n\nDone. Certificate available in file \"$PKIDIR/certs/${1}.crt\"."

